@@ -42,12 +42,17 @@ BOOL cmd_each(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         lf = kBel;
     }
 
-    if(command->mBlocksNum == 1) {
+    if(command->mBlocksNum == 1 && runinfo->mFilter) {
         fd_split(nextin, lf);
 
         char* argument;
         if(argument = sCommand_option_with_argument_item(command, "-number")) {
-            runinfo->mRCode = 0;
+            if(*(SFD(nextin).mBuf) == 0) {
+                runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+            }
+            else {
+                runinfo->mRCode = 0;
+            }
 
             int number = atoi(argument);
 
@@ -69,15 +74,26 @@ BOOL cmd_each(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
                 int rcode = 0;
                 if(!run(command->mBlocks[0], nextin2, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
-                    runinfo->mRCode = rcode;
-                    return FALSE;
+                    if(rcode == RCODE_BREAK) {
+                        runinfo->mRCode = rcode = 0;
+                        break;
+                    }
+                    else {
+                        runinfo->mRCode = rcode;
+                        return FALSE;
+                    }
                 }
 
                 runinfo->mRCode = rcode;
             }
         }
         else {
-            runinfo->mRCode = 0;
+            if(*(SFD(nextin).mBuf) == 0) {
+                runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+            }
+            else {
+                runinfo->mRCode = 0;
+            }
 
             sObject* nextin2 = FD_NEW_STACK();
 
@@ -93,8 +109,14 @@ BOOL cmd_each(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
                 int rcode = 0;
                 if(!run(command->mBlocks[0], nextin2, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
-                    runinfo->mRCode = rcode;
-                    return FALSE;
+                    if(rcode == RCODE_BREAK) {
+                        runinfo->mRCode = rcode = 0;
+                        break;
+                    }
+                    else {
+                        runinfo->mRCode = rcode;
+                        return FALSE;
+                    }
                 }
 
                 runinfo->mRCode = rcode;
@@ -166,7 +188,12 @@ BOOL cmd_join(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
         }
 
-        runinfo->mRCode = 0;
+        if(*(SFD(nextin).mBuf) == 0) {
+            runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+        }
+        else {
+            runinfo->mRCode = 0;
+        }
     }
 
     return TRUE;
@@ -190,8 +217,13 @@ BOOL cmd_lines(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         lf = kBel;
     }
 
-    if(command->mArgsNumRuntime > 1 && command->mBlocksNum == command->mArgsNumRuntime-1) {
-        runinfo->mRCode = 0;
+    if(command->mArgsNumRuntime > 1 && command->mBlocksNum == command->mArgsNumRuntime-1 && runinfo->mFilter) {
+        if(*(SFD(nextin).mBuf) == 0) {
+            runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+        }
+        else {
+            runinfo->mRCode = 0;
+        }
 
         fd_split(nextin, lf);
 
@@ -519,7 +551,12 @@ BOOL cmd_sort(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 }
             }
 
-            runinfo->mRCode = 0;
+            if(*(SFD(nextin).mBuf) == 0) {
+                runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+            }
+            else {
+                runinfo->mRCode = 0;
+            }
         } 
         else if(command->mBlocksNum == 1) {
             fd_split(nextin, lf);
@@ -540,8 +577,12 @@ BOOL cmd_sort(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     }
                 }
             }
-
-            runinfo->mRCode = 0;
+            if(*(SFD(nextin).mBuf) == 0) {
+                runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+            }
+            else {
+                runinfo->mRCode = 0;
+            }
         }
     }
 
