@@ -45,7 +45,14 @@ BOOL cmd_jobs(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             return FALSE;
         }
     }
-    runinfo->mRCode = 0;
+
+    if(vector_size(gJobs) == 0) {
+        runinfo->mRCode = 1;
+    }
+    else {
+        runinfo->mRCode = 0;
+    }
+
     return TRUE;
 }
 
@@ -266,7 +273,13 @@ BOOL cmd_stackframe(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         it = hash_loop_next(it);
     }
 
-    runinfo->mRCode = 0;
+    if(hash_count(SFUN(runinfo->mRunningObject).mLocalObjects) == 0) {
+        runinfo->mRCode = 1;
+    }
+    else {
+        runinfo->mRCode = 0;
+    }
+
     return TRUE;
 }
 
@@ -445,7 +458,13 @@ BOOL cmd_print(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 return FALSE;
             }
         }
-        runinfo->mRCode = 0;
+
+        if(*(SFD(nextin).mBuf) == 0) {
+            runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+        }
+        else {
+            runinfo->mRCode = 0;
+        }
     }
 
     return TRUE;
@@ -486,7 +505,7 @@ BOOL cmd_eval(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         int sline = 1;
         char buf2[BUFSIZ];
         snprintf(buf2, BUFSIZ, "%s %d: eval", runinfo->mSName, runinfo->mSLine);
-        if(parse(buf, buf2, &sline, block, NULL)) {
+        if(parse(buf, buf2, &sline, block, NULL, TRUE)) {
             if(!run(block, nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
                 err_msg_adding("run time error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = rcode;
@@ -512,7 +531,7 @@ BOOL cmd_eval(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         int sline = 1;
         char buf2[BUFSIZ];
         snprintf(buf2, BUFSIZ, "%s %d: eval", runinfo->mSName, runinfo->mSLine);
-        if(parse(buf, buf2, &sline, block, NULL)) {
+        if(parse(buf, buf2, &sline, block, NULL, TRUE)) {
             if(!run(block, nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
                 err_msg_adding("run time error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 runinfo->mRCode = rcode;
@@ -527,7 +546,13 @@ BOOL cmd_eval(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         }
 
         stack_end_stack();
-        runinfo->mRCode = rcode;
+
+        if(*(SFD(nextin).mBuf) == 0) {
+            runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+        }
+        else {
+            runinfo->mRCode = rcode;
+        }
     }
 
     return TRUE;
@@ -815,8 +840,7 @@ BOOL cmd_block(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             {
                 return FALSE;
             }
-
-            runinfo->mRCode = 0;
+            //runinfo->mRCode = runinfo->mRCode;
         }
     }
     else {
