@@ -620,12 +620,56 @@ BOOL cmd_condition_ef(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return TRUE;
 }
 
+void clear_matching_info_variable()
+{
+    char buf[128];
+    snprintf(buf, 128, "");
+    hash_put(gRootObject, "MATCH", STRING_NEW_GC(buf, FALSE));
+
+    snprintf(buf, 128, "");
+    hash_put(gRootObject, "PREMATCH", STRING_NEW_GC(buf, FALSE));
+
+    snprintf(buf, 128, "");
+    hash_put(gRootObject, "POSTMATCH", STRING_NEW_GC(buf, FALSE));
+
+    snprintf(buf, 128, "");
+    hash_put(gRootObject, "MATCH_NUMBER", STRING_NEW_GC(buf, FALSE));
+
+    snprintf(buf, 128, "");
+    hash_put(gRootObject, "LAST_MATCH", STRING_NEW_GC(buf, FALSE));
+
+    int i;
+    for(i=0; i<10; i++) {
+        char name[128];
+        snprintf(name, 128, "%d", i);
+        hash_put(gRootObject, name, STRING_NEW_GC(buf, FALSE));
+    }
+}
 
 BOOL cmd_condition_re(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     sCommand* command = runinfo->mCommand;
 
+    enum eKanjiCode code = gKanjiCode;
+    if(sCommand_option_item(command, "-byte")) {
+        code = kByte;
+    }
+    else if(sCommand_option_item(command, "-utf8")) {
+        code = kUtf8;
+    }
+    else if(sCommand_option_item(command, "-sjis")) {
+        code = kSjis;
+    }
+    else if(sCommand_option_item(command, "-eucjp")) {
+        code = kEucjp;
+    }
+
+    BOOL verbose = sCommand_option_item(command, "-verbose");
+    BOOL offsets = sCommand_option_item(command, "-offsets");
+
     if(runinfo->mFilter && command->mArgsNumRuntime == 2) {
+        clear_matching_info_variable();
+
         //BOOL preserve = sCommand_option_item(command, "-preserve");
 
         runinfo->mRCode = RCODE_NFUN_FALSE;
@@ -644,28 +688,21 @@ BOOL cmd_condition_re(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                , target, target + strlen(target)
                , region, ONIG_OPTION_NONE);
 
-            if(region->num_regs > 0) {
+            if(r2 >= 0) {
                 if(region->beg[0] > 0) {
-                    hash_put(SFUN(runinfo->mRunningObject).mLocalObjects, "PREMATCH", STRING_NEW_GC3(target, region->beg[0], FALSE));
-                }
-                else {
-                    hash_put(SFUN(runinfo->mRunningObject).mLocalObjects, "PREMATCH", STRING_NEW_GC("", FALSE));
+                    hash_put(gRootObject, "PREMATCH", STRING_NEW_GC3(target, region->beg[0], FALSE));
                 }
 
                 const int size = region->end[0] - region->beg[0];
 
-                hash_put(SFUN(runinfo->mRunningObject).mLocalObjects, "MATCH", STRING_NEW_GC3(target + region->beg[0], size, FALSE));
+                hash_put(gRootObject, "MATCH", STRING_NEW_GC3(target + region->beg[0], size, FALSE));
+                hash_put(gRootObject, "0", STRING_NEW_GC3(target + region->beg[0], size, FALSE));
 
                 const int n = strlen(target)-region->end[0];
                 if(n > 0) {
-                    hash_put(SFUN(runinfo->mRunningObject).mLocalObjects, "POSTMATCH", STRING_NEW_GC3(target + region->end[0], n, FALSE));
+                    hash_put(gRootObject, "POSTMATCH", STRING_NEW_GC3(target + region->end[0], n, FALSE));
                 }
-                else {
-                    hash_put(SFUN(runinfo->mRunningObject).mLocalObjects, "POSTMATCH", STRING_NEW_GC("", FALSE));
-                }
-            }
 
-            if(r2 >= 0) {
                 int i;
                 for (i=1; i<region->num_regs; i++) {
                     const int size = region->end[i] - region->beg[i];
@@ -673,11 +710,63 @@ BOOL cmd_condition_re(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     char name[16];
                     snprintf(name, 16, "%d", i);
 
-                    hash_put(SFUN(runinfo->mRunningObject).mLocalObjects, name, STRING_NEW_GC3(target + region->beg[i], size, FALSE));
+                    hash_put(gRootObject, name, STRING_NEW_GC3(target + region->beg[i], size, FALSE));
+                }
+
+                if(region->num_regs > 0) {
+                    const int n = region->num_regs -1;
+
+                    const int size = region->end[n] - region->beg[n];
+
+                    hash_put(gRootObject, "LAST_MATCH", STRING_NEW_GC3(target + region->beg[n], size, FALSE));
+                }
+
+                char buf[128];
+                snprintf(buf, 128, "%d", region->num_regs);
+                hash_put(gRootObject, "MATCH_NUMBER", STRING_NEW_GC(buf, FALSE));
+
+                if(verbose) {
+                    int point = str_pointer2kanjipos(code, target, target + r2);
+
+                    char buf[1024];
+                    snprintf(buf, 1024, "%d\n", point);
+                    if(!fd_write(nextout, buf)) {
+                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                        onig_region_free(region, 1);
+                        onig_free(reg);
+                        return FALSE;
+                    }
+                }
+
+                if(offsets) {
+                    int i;
+                    for (i=0; i<region->num_regs; i++) {
+                        int point = str_pointer2kanjipos(code, target, target + region->beg[i]);
+
+                        char buf[1024];
+                        snprintf(buf, 1024, "%d\n", point);
+                        if(!fd_write(nextout, buf)) {
+                            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                            onig_region_free(region, 1);
+                            onig_free(reg);
+                            return FALSE;
+                        }
+
+                        point = str_pointer2kanjipos(code, target, target + region->end[i]);
+
+                        snprintf(buf, 1024, "%d\n", point);
+                        if(!fd_write(nextout, buf)) {
+                            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                            onig_region_free(region, 1);
+                            onig_free(reg);
+                            return FALSE;
+                        }
+                    }
                 }
-            }
 
-            if(r2 >= 0) {
                 runinfo->mRCode = 0;
             }
 
@@ -694,3 +783,150 @@ BOOL cmd_condition_re(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return TRUE;
 }
 
+#if defined(HAVE_MIGEMO_H)
+#include <migemo.h>
+
+migemo* gMigemo;
+static regex_t* gReg;
+
+void migemo_init()
+{
+    char buf[PATH_MAX];
+    char migemodir[PATH_MAX];
+    gMigemo = migemo_open(NULL);
+
+    snprintf(migemodir, PATH_MAX, "%s", SYSTEM_MIGEMODIR);
+
+    snprintf(buf, PATH_MAX, "%s/utf-8/migemo-dict", migemodir);
+    if(migemo_load(gMigemo, MIGEMO_DICTID_MIGEMO, buf) == MIGEMO_DICTID_INVALID) {
+        fprintf(stderr, "%s is not found\n", buf);
+        exit(1);
+    }
+    snprintf(buf, PATH_MAX, "%s/utf-8/roma2hira.dat", migemodir);
+    if(migemo_load(gMigemo, MIGEMO_DICTID_ROMA2HIRA, buf) == MIGEMO_DICTID_INVALID) {
+        fprintf(stderr, "%s is not found\n", buf);
+        exit(1);
+    }
+    snprintf(buf, PATH_MAX, "%s/utf-8/hira2kata.dat", migemodir);
+    if(migemo_load(gMigemo, MIGEMO_DICTID_HIRA2KATA, buf) == MIGEMO_DICTID_INVALID) {
+        fprintf(stderr, "%s is not found\n", buf);
+        exit(1);
+    }
+    snprintf(buf, PATH_MAX, "%s/utf-8/han2zen.dat", migemodir);
+    if(migemo_load(gMigemo, MIGEMO_DICTID_HAN2ZEN, buf) == MIGEMO_DICTID_INVALID) {
+        fprintf(stderr, "%s is not found\n", buf);
+        exit(1);
+    }
+}
+
+void migemo_final()
+{
+    onig_end();
+    migemo_close(gMigemo);
+    if(gReg) onig_free(gReg);
+    gReg = NULL;
+}
+
+BOOL cmd_migemo_match(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    sCommand* command = runinfo->mCommand;
+
+    BOOL quiet = sCommand_option_item(command, "-quiet");
+
+    if(runinfo->mFilter && command->mArgsNumRuntime == 2) {
+        runinfo->mRCode = RCODE_NFUN_FALSE;
+        char* target = SFD(nextin).mBuf;
+        char* regex = command->mArgsRuntime[1];
+
+        if(regex[0] == 0) {
+            runinfo->mRCode = 0;
+            if(!quiet) {
+                char buf[1024];
+                snprintf(buf, 1024, "0%d\n", (int)strlen(target));
+                if(!fd_write(nextout, buf)) {
+                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                    return FALSE;
+                }
+            }
+        }
+        else {
+            OnigUChar * p = migemo_query(gMigemo, regex);
+            if(p == NULL) {
+                err_msg("migemo query failed", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                migemo_release(gMigemo, (unsigned char*) p);
+                return FALSE;
+            }
+
+            /// modify query ///
+            char* p2 = MALLOC(strlen(p)*2 + 1);
+
+            char* _p = p;
+            char* _p2 = p2;
+
+            while(*_p) {
+                if(*_p == '+') {
+                    *_p2++ = '\\';
+                    *_p2++ = *_p++;
+                }
+                else {
+                    *_p2++ = *_p++;
+                }
+            }
+            *_p2 = 0;
+
+            /// make regex ///
+            regex_t* reg;
+            OnigErrorInfo err_info;
+
+            int r = onig_new(&reg, p2, p2 + strlen(p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_UTF8, ONIG_SYNTAX_DEFAULT,  &err_info);
+
+            FREE(p2);
+            migemo_release(gMigemo, (unsigned char*) p);
+
+            if(r != ONIG_NORMAL && r != 0) {
+                err_msg("regex of migemo query failed", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                onig_free(reg);
+                return FALSE;
+            }
+
+            OnigRegion* region = onig_region_new();
+            int r2 = onig_search(reg, target, target + strlen(target), target, target + strlen(target), region, ONIG_OPTION_NONE);
+
+            if(r2 == 0) {
+                runinfo->mRCode = 0;
+                if(!quiet) {
+                    char buf[1024];
+                    snprintf(buf, 1024, "%d\n%d\n", region->beg[0], region->end[0]);
+                    if(!fd_write(nextout, buf)) {
+                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                        onig_region_free(region, 1);
+                        onig_free(reg);
+                        return FALSE;
+                    }
+                }
+            }
+            else {
+                if(!quiet) {
+                    char buf[1024];
+                    snprintf(buf, 1024, "-1\n-1\n");
+                    if(!fd_write(nextout, buf)) {
+                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                        onig_region_free(region, 1);
+                        onig_free(reg);
+                        return FALSE;
+                    }
+                }
+            }
+
+            onig_region_free(region, 1);
+            onig_free(reg);
+        }
+    }
+
+    return TRUE;
+}
+
+#endif
