@@ -286,7 +286,12 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                             return FALSE;
                         }
                     }
-                    runinfo->mRCode = 0;
+                    if(*(SFD(nextin).mBuf) == 0) {
+                        runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+                    }
+                    else {
+                        runinfo->mRCode = 0;
+                    }
                     break;
                 }
 
@@ -326,6 +331,9 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 #endif
             mrestore_ttysettings();
         }
+        else {
+            runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+        }
     }
 
     return TRUE;
@@ -520,7 +528,12 @@ BOOL cmd_p(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     FREE(markfiles);
                     return FALSE;
                 }
-                runinfo->mRCode = 0;
+                if(*(SFD(nextin).mBuf) == 0) {
+                    runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+                }
+                else {
+                    runinfo->mRCode = 0;
+                }
                 break;
             }
 
