@@ -28,7 +28,7 @@ BOOL cmd_write(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     sCommand* command = runinfo->mCommand;
 
-    if(command->mArgsNumRuntime == 2) {
+    if(command->mArgsNumRuntime == 2 && runinfo->mFilter) {
         char* fname = command->mArgsRuntime[1];
 
         int fd;
@@ -65,7 +65,12 @@ BOOL cmd_write(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
         (void)close(fd);
 
-        runinfo->mRCode = 0;
+        if(*(SFD(nextin).mBuf) == 0) {
+            runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+        }
+        else {
+            runinfo->mRCode = 0;
+        }
     }
 
     return TRUE;
@@ -246,9 +251,11 @@ BOOL cmd_popd(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     if(vector_size(gDirStack) > 0) {
         sObject* pwd = vector_pop_back(gDirStack);
 
-        setenv("PWD", string_c_str(pwd), 1);
-        if(chdir(string_c_str(pwd)) >= 0) {
-            runinfo->mRCode = 0;
+        if(pwd) {
+            setenv("PWD", string_c_str(pwd), 1);
+            if(chdir(string_c_str(pwd)) >= 0) {
+                runinfo->mRCode = 0;
+            }
         }
     }
 
