@@ -96,7 +96,12 @@ BOOL cmd_plus(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             return FALSE;
         }
 
-        runinfo->mRCode = 0;
+        if(*(SFD(nextin).mBuf) == 0) {
+            runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+        }
+        else {
+            runinfo->mRCode = 0;
+        }
     }
 
     return TRUE;
@@ -122,7 +127,12 @@ BOOL cmd_minus(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             return FALSE;
         }
 
-        runinfo->mRCode = 0;
+        if(*(SFD(nextin).mBuf) == 0) {
+            runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+        }
+        else {
+            runinfo->mRCode = 0;
+        }
     }
 
     return TRUE;
@@ -148,7 +158,12 @@ BOOL cmd_mult(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             return FALSE;
         }
 
-        runinfo->mRCode = 0;
+        if(*(SFD(nextin).mBuf) == 0) {
+            runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+        }
+        else {
+            runinfo->mRCode = 0;
+        }
     }
 
     return TRUE;
@@ -181,7 +196,12 @@ BOOL cmd_div(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             return FALSE;
         }
 
-        runinfo->mRCode = 0;
+        if(*(SFD(nextin).mBuf) == 0) {
+            runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+        }
+        else {
+            runinfo->mRCode = 0;
+        }
     }
 
     return TRUE;
@@ -207,7 +227,12 @@ BOOL cmd_mod(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             return FALSE;
         }
 
-        runinfo->mRCode = 0;
+        if(*(SFD(nextin).mBuf) == 0) {
+            runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+        }
+        else {
+            runinfo->mRCode = 0;
+        }
     }
 
     return TRUE;
@@ -233,7 +258,12 @@ BOOL cmd_pow(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             return FALSE;
         }
 
-        runinfo->mRCode = 0;
+        if(*(SFD(nextin).mBuf) == 0) {
+            runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+        }
+        else {
+            runinfo->mRCode = 0;
+        }
     }
 
     return TRUE;
@@ -257,7 +287,12 @@ BOOL cmd_abs(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             return FALSE;
         }
 
-        runinfo->mRCode = 0;
+        if(*(SFD(nextin).mBuf) == 0) {
+            runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+        }
+        else {
+            runinfo->mRCode = 0;
+        }
     }
 
     return TRUE;
