@@ -71,14 +71,13 @@ static BOOL entry_fun(sObject* nextin, sObject* nextout, sRunInfo* runinfo, sObj
                 return FALSE;
             }
 
-            if(TYPE(parent) == T_FUN || TYPE(parent) == T_NFUN) {
-                SFUN(block).mParent = parent;
-                SFUN(block).mParentCount = SFUN(parent).mParentCount + 1;
-            }
-            else {
+            if(type == T_CLASS && (TYPE(parent) == T_FUN || TYPE(parent) == T_NFUN) || type == T_FUN && TYPE(parent) == T_CLASS) {
                 err_msg("can't inherit a parent function beacuse of the different type", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 return FALSE;
             }
+
+            SFUN(block).mParent = parent;
+            SFUN(block).mParentCount = SFUN(parent).mParentCount + 1;
         }
         else {
             err_msg("can't inherit. There is not a parent", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
@@ -155,7 +154,7 @@ BOOL cmd_def(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             int rcode = 0;
             sObject* block = BLOCK_NEW_STACK();
             int sline = 1;
-            if(!parse(SFD(nextin).mBuf, "def", &sline, block, NULL)) {
+            if(!parse(SFD(nextin).mBuf, "def", &sline, block, NULL, TRUE)) {
                 err_msg("parser error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 stack_end_stack();
                 return FALSE;
@@ -167,7 +166,12 @@ BOOL cmd_def(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
             stack_end_stack();
             
-            runinfo->mRCode = 0;
+            if(*(SFD(nextin).mBuf) == 0) {
+                runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+            }
+            else {
+                runinfo->mRCode = 0;
+            }
         }
         /// output ///
         else {
@@ -218,7 +222,7 @@ BOOL cmd_class(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             int rcode = 0;
             sObject* block = BLOCK_NEW_STACK();
             int sline = 1;
-            if(!parse(SFD(nextin).mBuf, "def", &sline, block, NULL)) {
+            if(!parse(SFD(nextin).mBuf, "def", &sline, block, NULL, TRUE)) {
                 err_msg("parser error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 stack_end_stack();
                 return FALSE;
@@ -230,7 +234,12 @@ BOOL cmd_class(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
             stack_end_stack();
             
-            runinfo->mRCode = 0;
+            if(*(SFD(nextin).mBuf) == 0) {
+                runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+            }
+            else {
+                runinfo->mRCode = 0;
+            }
         }
         /// output ///
         else {
@@ -290,6 +299,7 @@ BOOL cmd_inherit(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         err_msg("invalid command using", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                         return FALSE;
                     }
+                    //runinfo->mRCode = 0;
                 }
                 else {  // T_FUN
                     sCommand* command = runinfo->mCommand;
@@ -297,11 +307,9 @@ BOOL cmd_inherit(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     {
                         return FALSE;
                     }
+                    //runinfo->mRCode = 0;
                 }
             }
-
-            runinfo->mRCode = 0;
-
             }
             break;
 
@@ -314,11 +322,8 @@ BOOL cmd_inherit(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 {
                     return FALSE;
                 }
-
+                //runinfo->mRCode = 0;
             }
-
-            runinfo->mRCode = 0;
-
             }
             break;
 
@@ -482,7 +487,12 @@ BOOL cmd_var(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 }
             }
 
-            runinfo->mRCode = 0;
+            if(*(SFD(nextin).mBuf) == 0) {
+                runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+            }
+            else {
+                runinfo->mRCode = 0;
+            }
         }
     }
 
@@ -550,13 +560,13 @@ BOOL cmd_mrun(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             if(!run_function(klass, nextin, nextout, runinfo, command->mArgsRuntime + 2, command->mArgsNumRuntime -2, command->mBlocks, command->mBlocksNum)) {
                 return FALSE;
             }
+
+            //runinfo->mRCode = 0;
         }
         else {
             err_msg("not found variable", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             return FALSE;
         }
-
-        runinfo->mRCode = 0;
     }
 
     return TRUE;
@@ -698,13 +708,19 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                         return FALSE;
                     }
-                    runinfo->mRCode = 0;
                 }
                 else {
                     err_msg("invalid adress. it's not memory managed by a pipes gabage collecter", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                     return FALSE;
                 }
             }
+
+            if(*(SFD(nextin).mBuf) == 0) {
+                runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+            }
+            else {
+                runinfo->mRCode = 0;
+            }
         }
         else {
             if(command->mArgsNumRuntime > 1) {
@@ -721,7 +737,6 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
                         if(type >= 0) {
                             hash_put(object, command->mArgsRuntime[i+1], mem);
-                            runinfo->mRCode = 0;
                         }
                         else {
                             err_msg("invalid adress. it's not memory managed by a pipes gabage collecter", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
@@ -739,6 +754,13 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         break;
                     }
                 }
+
+                if(*(SFD(nextin).mBuf) == 0) {
+                    runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+                }
+                else {
+                    runinfo->mRCode = 0;
+                }
             }
         }
     }
@@ -907,7 +929,13 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
             vector_mass_insert(ary, append, appended_ary, max);
             FREE(appended_ary);
-            runinfo->mRCode = 0;
+            
+            if(*(SFD(nextin).mBuf) == 0) {
+                runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+            }
+            else {
+                runinfo->mRCode = 0;
+            }
         }
         else {
             sObject* object;
@@ -927,7 +955,13 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 string_chomp(new_var);
                 vector_add(new_ary, new_var);
             }
-            runinfo->mRCode = 0;
+            
+            if(*(SFD(nextin).mBuf) == 0) {
+                runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+            }
+            else {
+                runinfo->mRCode = 0;
+            }
         }
     }
 
@@ -1094,7 +1128,13 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     hash_put(hash, string_c_str(key), new_var);
                 }
             }
-            runinfo->mRCode = 0;
+                
+            if(*(SFD(nextin).mBuf) == 0) {
+                runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+            }
+            else {
+                runinfo->mRCode = 0;
+            }
         }
         else {
             sObject* object;
@@ -1121,7 +1161,13 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     hash_put(new_hash, string_c_str(key), new_var);
                 }
             }
-            runinfo->mRCode = 0;
+                
+            if(*(SFD(nextin).mBuf) == 0) {
+                runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+            }
+            else {
+                runinfo->mRCode = 0;
+            }
         }
     }
 
@@ -1203,7 +1249,12 @@ BOOL cmd_export(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 }
             }
 
-            runinfo->mRCode = 0;
+            if(*(SFD(nextin).mBuf) == 0) {
+                runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
+            }
+            else {
+                runinfo->mRCode = 0;
+            }
         }
     }
 
