@@ -175,7 +175,7 @@ void pipes_readline_interface(char** argv, int argc, BOOL no_runtime_script)
             int rcode = 0;
             sObject* block = BLOCK_NEW_STACK();
             int sline = 1;
-            if(parse(buf, "pipes", &sline, block, NULL)) {
+            if(parse(buf, "pipes", &sline, block, NULL, TRUE)) {
                 pipes_set_signal();
 
                 sObject* fun = FUN_NEW_STACK(NULL);
@@ -304,7 +304,7 @@ void pipes_opt_c(char* cmd, char** argv, int argc, BOOL no_runtime_script)
     int rcode = 0;
     sObject* block = BLOCK_NEW_STACK();
     int sline = 1;
-    if(parse(cmd, "pipes", &sline, block, NULL)) {
+    if(parse(cmd, "pipes", &sline, block, NULL, TRUE)) {
         pipes_set_signal_optc();
 
         sObject* fun = FUN_NEW_STACK(NULL);
