@@ -91,6 +91,7 @@ void uobject_root_init(sObject* self)
     hash_put(self, "lc", NFUN_NEW_GC(cmd_lc, NULL, TRUE));
     hash_put(self, "uc", NFUN_NEW_GC(cmd_uc, NULL, TRUE));
     hash_put(self, "chomp", NFUN_NEW_GC(cmd_chomp, NULL, TRUE));
+    hash_put(self, "chop", NFUN_NEW_GC(cmd_chop, NULL, TRUE));
     hash_put(self, "pomch", NFUN_NEW_GC(cmd_pomch, NULL, TRUE));
     hash_put(self, "funinfo", NFUN_NEW_GC(cmd_funinfo, NULL, TRUE));
     hash_put(self, "printf", NFUN_NEW_GC(cmd_printf, NULL, TRUE));
@@ -200,6 +201,8 @@ void uobject_root_init(sObject* self)
     nfun = NFUN_NEW_GC(cmd_class, NULL, TRUE);
     (void)nfun_put_option_with_argument(nfun, STRDUP("-option-with-argument"));
     hash_put(self, "class", nfun);
+
+    clear_matching_info_variable();
 }
 
 void uobject_init(sObject* self)
