@@ -113,10 +113,10 @@ static BOOL read_quote(char**p, sBuf* buf, char* sname, int* sline)
     return TRUE;
 }
 
-static BOOL block_parse(char** p, char* sname, int* sline, sObject* block, sObject** current_object);
-static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_quote, sCommand* command, sObject* block, sObject** current_object);
+static BOOL block_parse(char** p, char* sname, int* sline, sObject* block, sObject** current_object, BOOL tilda_expand);
+static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_quote, sCommand* command, sObject* block, sObject** current_object, BOOL tilda_expand);
 
-static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_quote, sCommand* command, sObject* block, sObject** current_object)
+static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_quote, sCommand* command, sObject* block, sObject** current_object, BOOL tilda_expand)
 {
     (*p)++;
 
@@ -165,11 +165,10 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
         (*p)++;
 
         sObject* block2 = BLOCK_NEW_STACK();
-        if(!block_parse(p, sname, sline, block2, current_object)) {
-            sCommand_add_env_block(command, block2, lf);
-
+        if(!block_parse(p, sname, sline, block2, current_object, tilda_expand)) {
             SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_ENV_BLOCK;
             SBLOCK(block).mCompletionFlags |= command->mEnvsNum & 0xFF;
+            sCommand_add_env_block(command, block2, lf);
             return FALSE;
         }
 
@@ -229,7 +228,7 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
         if(**p == '[') {
             (*p)++;
 
-            if(!read_one_argument(p, &key, sname, sline, TRUE, command, block, current_object)) 
+            if(!read_one_argument(p, &key, sname, sline, TRUE, command, block, current_object, tilda_expand)) 
             {
                 FREE(name.mBuf);
                 if(key.mEnv) {  // for readline
@@ -266,7 +265,7 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
 }
 
 /// ñɤ߹
-static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_quote, sCommand* command, sObject* block, sObject** current_object)
+static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_quote, sCommand* command, sObject* block, sObject** current_object, BOOL tilda_expand)
 {
     /// argument separated character table
     static unsigned char table[] = { 
@@ -309,7 +308,7 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
         }
         /// env ///
         else if(!squote && **p == '$') {
-            if(!read_env(p, buf, sname,sline, expand_quote, command, block, current_object)) {
+            if(!read_env(p, buf, sname,sline, expand_quote, command, block, current_object, tilda_expand)) {
                 return FALSE;
             }
         }
@@ -395,7 +394,7 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
             while(**p) {
                 /// env ///
                 if(!squote && **p == '$') {
-                    if(!read_env(p, buf, sname,sline, expand_quote, command, block, current_object)) {
+                    if(!read_env(p, buf, sname,sline, expand_quote, command, block, current_object, tilda_expand)) {
                         return FALSE;
                     }
                 }
@@ -455,7 +454,6 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
         else if(command->mArgsNum == 0 && **p == '-' && *(*p+1) == '>') {
             (*p)+=2;
             buf->mMessagePassing = TRUE;
-            SBLOCK(block).mCompletionFlags &= ~(COMPLETION_FLAGS_NEXT_STATMENT|COMPLETION_FLAGS_NEXT_PIPE);
             break;
         }
         /// utf-8 character
@@ -466,6 +464,10 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
         else if(table[**p]) {
             break;
         }
+        else if(**p == '/') {
+            add_char_to_buf(buf, **p);
+            (*p)++;
+        }
         else {
             add_char_to_buf(buf, **p);
             (*p)++;
@@ -480,7 +482,7 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
     return TRUE;
 }
 
-static void tilda_expasion(char* file, ALLOC char** file2)
+void tilda_expasion(char* file, ALLOC char** file2)
 {
     char* p = file;
     p++; // ~
@@ -539,11 +541,11 @@ static void tilda_expasion(char* file, ALLOC char** file2)
     *file2 = STRDUP(result);
 }
 
-static BOOL glob_expasion(sBuf* buf, sCommand* command, char* sname, int* sline)
+static BOOL glob_expasion(sBuf* buf, sCommand* command, char* sname, int* sline, BOOL tilda_expand)
 {
     glob_t result;
     int rc;
-    if(buf->mTilda) {  // tilda expasion
+    if(buf->mTilda && tilda_expand) {  // tilda expasion
         char* buf2;
         tilda_expasion(buf->mBuf, ALLOC &buf2);
 
@@ -571,11 +573,11 @@ static BOOL glob_expasion(sBuf* buf, sCommand* command, char* sname, int* sline)
     return TRUE;
 }
 
-static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sname, int* sline, sObject** current_object);
+static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sname, int* sline, sObject** current_object, BOOL tilda_expand);
 
 // result --> TRUE: success
 // result --> FALSE: There is a error message on gErrMsg. Please output the error message and stop running
-static BOOL block_parse(char** p, char* sname, int* sline, sObject* block, sObject** current_object)
+static BOOL block_parse(char** p, char* sname, int* sline, sObject* block, sObject** current_object, BOOL tilda_expand)
 {
     char* source_head = *p;
 
@@ -589,7 +591,7 @@ static BOOL block_parse(char** p, char* sname, int* sline, sObject* block, sObje
         /// go parsing ///
         sStatment* statment = sStatment_new(block, *sline, sname);
 
-        if(!read_statment(p, statment, block, sname, sline, current_object))
+        if(!read_statment(p, statment, block, sname, sline, current_object, tilda_expand))
         {
             return FALSE;
         }
@@ -621,10 +623,10 @@ static BOOL block_parse(char** p, char* sname, int* sline, sObject* block, sObje
     return TRUE;
 }
 
-static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObject* block, char* sname, int* sline, sObject** current_object)
+static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObject* block, char* sname, int* sline, sObject** current_object, BOOL tilda_expand)
 {
     while(**p) {
-        SBLOCK(block).mCompletionFlags &= ~COMPLETION_FLAGS_WORDBREAK_CHARACTER;
+        SBLOCK(block).mCompletionFlags &= ~(COMPLETION_FLAGS_LAST_CHAR_IS_SPACE|COMPLETION_FLAGS_ENV|COMPLETION_FLAGS_ENV_BLOCK|COMPLETION_FLAGS_TILDA|COMPLETION_FLAGS_BLOCK);
 
         sBuf buf;
         memset(&buf, 0, sizeof(sBuf));
@@ -632,7 +634,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObje
         buf.mBuf[0] = 0;
         buf.mSize = 64;
 
-        if(!read_one_argument(p, &buf, sname, sline, TRUE, command, block, current_object)) 
+        if(!read_one_argument(p, &buf, sname, sline, TRUE, command, block, current_object, tilda_expand)) 
         {
             if(buf.mEnv) {  // for readline
                 sCommand_add_arg_to_command(command, MANAGED buf.mBuf, TRUE);
@@ -659,14 +661,14 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObje
         }
         /// expand glob ///
         else if(buf.mGlob && command->mArgsNum > 0) {
-            if(!glob_expasion(&buf, command, sname, sline)) {
+            if(!glob_expasion(&buf, command, sname, sline, tilda_expand)) {
                 FREE(buf.mBuf);
                 return FALSE;
             }
             FREE(buf.mBuf);
         }
         /// tilda expression ///
-        else if(buf.mTilda) {
+        else if(buf.mTilda && tilda_expand) {
             char* buf2;
             tilda_expasion(buf.mBuf, ALLOC &buf2);
             sCommand_add_arg_to_command(command, MANAGED buf2, 0);
@@ -680,20 +682,14 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObje
             FREE(buf.mBuf);
         }
 
-        if(**p != 0) {
-            SBLOCK(block).mCompletionFlags &= ~COMPLETION_FLAGS_ENV;
-            SBLOCK(block).mCompletionFlags &= ~COMPLETION_FLAGS_TILDA;
-        }
-
         /// subshell, block ///
         if(**p == '(') {
             (*p)++;
 
             if(command->mArgsNum == 0) {
                 sObject* block2 = BLOCK_NEW_STACK();
-                if(!block_parse(p, sname, sline, block2, current_object)) {
+                if(!block_parse(p, sname, sline, block2, current_object, tilda_expand)) {
                     sCommand_add_block_to_command(command, block2);
-
                     SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_BLOCK;
                     SBLOCK(block).mCompletionFlags |= command->mBlocksNum & 0xFF;
                     return FALSE;
@@ -731,9 +727,8 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObje
                 }
 
                 sObject* block2 = BLOCK_NEW_STACK();
-                if(!block_parse(p, sname, sline, block2, current_object)) {
+                if(!block_parse(p, sname, sline, block2, current_object, tilda_expand)) {
                     sCommand_add_block_to_command(command, block2);
-
                     SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_BLOCK;
                     SBLOCK(block).mCompletionFlags |= command->mBlocksNum & 0xFF;
                     return FALSE;
@@ -749,7 +744,9 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObje
         }
         /// spaces ///
         else if(**p == ' ' || **p == '\t') {
-            SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_WORDBREAK_CHARACTER;
+            SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_LAST_CHAR_IS_SPACE;
+            SBLOCK(block).mCompletionFlags &= ~COMPLETION_FLAGS_ENV;
+            SBLOCK(block).mCompletionFlags &= ~COMPLETION_FLAGS_TILDA;
             skip_spaces(p);
         }
         else {
@@ -763,7 +760,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObje
 /// read statment
 /// TRUE: OK
 /// FALSE: indicate to occure error
-static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sname, int* sline, sObject** current_object)
+static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sname, int* sline, sObject** current_object, BOOL tilda_expand)
 {
     /// a reversing code ///
     if(**p == '!') {
@@ -794,10 +791,8 @@ static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sn
             return FALSE;
         }
 
-        SBLOCK(block).mCompletionFlags &= ~COMPLETION_FLAGS_NEXT_PIPE;
-
         sCommand* command = sCommand_new(statment);
-        if(!read_command(p, command, statment, block, sname, sline, current_object)) {
+        if(!read_command(p, command, statment, block, sname, sline, current_object, tilda_expand)) {
             return FALSE;
         }
 
@@ -819,10 +814,15 @@ static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sn
         {
             (*p)++;
 
-            SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_NEXT_PIPE;
             skip_spaces(p);
+
+            if(**p == 0) {
+                SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_LAST_CHAR_IS_PIPE;
+                break;
+            }
         }
         else {
+            SBLOCK(block).mCompletionFlags &= ~COMPLETION_FLAGS_LAST_CHAR_IS_PIPE;
             break;
         }
     }
@@ -841,24 +841,18 @@ static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sn
         skip_spaces(p);
 
         statment->mFlags |= STATMENT_ANDAND;
-
-        SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_NEXT_STATMENT;
     }
     else if(**p == '|' && *(*p+1) == '|') {
         (*p)+=2;
         skip_spaces(p);
 
         statment->mFlags |= STATMENT_OROR;
-
-        SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_NEXT_STATMENT;
     }
     else if(**p == '&') {
         (*p)++;
         skip_spaces(p);
 
         statment->mFlags |= STATMENT_BACKGROUND;
-
-        SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_NEXT_STATMENT;
     }
     else if(**p == '\n') {
         (*p)++; 
@@ -866,16 +860,12 @@ static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sn
 
         statment->mFlags |= STATMENT_NORMAL;
         (*sline)++;
-
-        SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_NEXT_STATMENT;
     }
     else if(**p == ';') {
         (*p)++;
         skip_spaces(p);
 
         statment->mFlags |= STATMENT_NORMAL;
-
-        SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_NEXT_STATMENT;
     }
     else if(**p == 0) {
         statment->mFlags |= STATMENT_NORMAL;
@@ -894,7 +884,7 @@ static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sn
 
 // result --> TRUE: success
 // result --> FALSE: There is a error message on gErrMsg. Please output the error message and stop running
-BOOL parse(char* p, char* sname, int* sline, sObject* block, sObject** current_object)
+BOOL parse(char* p, char* sname, int* sline, sObject* block, sObject** current_object, BOOL tilda_expand)
 {
     SBLOCK(block).mSource = STRDUP(p);
 
@@ -906,11 +896,9 @@ BOOL parse(char* p, char* sname, int* sline, sObject* block, sObject** current_o
         if(*p == '#') skip_to_nextline(&p, sline);
 
         /// go parsing ///
-        SBLOCK(block).mCompletionFlags &= ~(COMPLETION_FLAGS_NEXT_STATMENT);
-
         sStatment* statment = sStatment_new(block, *sline, sname);
 
-        if(!read_statment(&p, statment, block, sname, sline, current_object))
+        if(!read_statment(&p, statment, block, sname, sline, current_object, tilda_expand))
         {
             return FALSE;
         }
