@@ -108,8 +108,7 @@ static void pipes_read_rc()
     pipes_restore_signal_default();
 
 
-    hash_put(gRootObject, "test", sTestObj_new("AAA", 1));
-
+//    hash_put(gRootObject, "test", sTestObj_new("AAA", 1));
 }
 
 void pipes_init(enum eAppType app_type, BOOL no_runtime_script)
