@@ -217,9 +217,8 @@ typedef struct {
     int mFlags;
 } sStatment;
 
-#define COMPLETION_FLAGS_WORDBREAK_CHARACTER 0x0100
-#define COMPLETION_FLAGS_NEXT_PIPE 0x0200
-#define COMPLETION_FLAGS_NEXT_STATMENT 0x0400
+#define COMPLETION_FLAGS_LAST_CHAR_IS_SPACE 0x0100
+#define COMPLETION_FLAGS_LAST_CHAR_IS_PIPE 0x0200
 #define COMPLETION_FLAGS_ENV 0x0800
 #define COMPLETION_FLAGS_ENV_BLOCK 0x1000
 #define COMPLETION_FLAGS_BLOCK 0x2000
@@ -512,6 +511,7 @@ BOOL cmd_rindex(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_uc(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_lc(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_chomp(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
+BOOL cmd_chop(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_pomch(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_printf(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_join(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
@@ -620,7 +620,8 @@ int gc_sweep();
 char* xstrncpy(char* src, char* des, int size);
 char* xstrncat(char* src, char* des, int size);
 
-BOOL parse(char* p, char* sname, int* sline, sObject* block, sObject** current_object);
+BOOL parse(char* p, char* sname, int* sline, sObject* block, sObject** current_object, BOOL tilda_expand);
+void tilda_expasion(char* file, ALLOC char** file2);
 
 #define RCODE_BREAK 256
 #define RCODE_RETURN 257
@@ -628,6 +629,7 @@ BOOL parse(char* p, char* sname, int* sline, sObject* block, sObject** current_o
 #define RCODE_SIGNAL_INTERRUPT 259
 #define RCODE_NFUN_FALSE 260
 #define RCODE_NFUN_INVALID_USSING 261
+#define RCODE_NFUN_NULL_INPUT 262
 //#define RCODE_COMMAND_NOT_FOUND 262
 
 void pipes_kill_all_jobs();
@@ -723,5 +725,6 @@ void migemo_final();
 #endif
 
 int readline_signal();
+void clear_matching_info_variable();
 
 #endif
