@@ -126,6 +126,63 @@ static char* message_completion(const char* text, int stat)
     return NULL;
 }
 
+static char* all_program_completion(const char* text, int stat)
+{
+    static int index, wordlen;
+
+    if(stat == 0) {
+        gCompletionArray = VECTOR_NEW_STACK(16);
+        sObject* hash = HASH_NEW_STACK(16);
+
+        sObject* current = gReadlineCurrentObject;
+        while(1) {
+            hash_it* it = hash_loop_begin(current);
+            while(it) {
+                char* key = hash_loop_key(it);
+                sObject* object = hash_loop_item(it);
+
+                if(hash_item(hash, key) == NULL) {
+                    hash_put(hash, key, object);
+
+                    sObject* candidate = STRING_NEW_STACK(hash_loop_key(it));
+                    if(TYPE(object) == T_UOBJECT) {
+                        if(hash_item(object, "main")) {
+                            vector_add(gCompletionArray, string_c_str(candidate));
+                        }
+                        else {
+                            sObject* candidate2 = STRING_NEW_STACK(hash_loop_key(it));
+                            string_push_back(candidate2, "->");
+                            vector_add(gCompletionArray, string_c_str(candidate2));
+                        }
+                    }
+                    else {
+                        vector_add(gCompletionArray, string_c_str(candidate));
+                    }
+                }
+
+                it = hash_loop_next(it);
+            }
+            
+            if(current == gRootObject) break;
+
+            current = SUOBJECT(current).mParent;
+        }
+        vector_sort(gCompletionArray, name_sort);
+
+        wordlen = strlen(text);
+        index = 0;
+    }
+
+    while(index < vector_size(gCompletionArray)) {
+        char* candidate = vector_item(gCompletionArray, index);
+        index++;
+
+        return strdup(candidate);
+    }
+
+    return NULL;
+}
+
 static char* program_completion(const char* text, int stat)
 {
     static int index, wordlen;
@@ -274,53 +331,31 @@ static char* user_completion(const char* text, int stat)
     return NULL;
 }
 
-/*
 static char* tilda_completion(const char* text, int stat)
 {
     static int index, wordlen;
+    static sObject* pw_dirs = NULL;
 
     if(stat == 0) {
         gCompletionArray = VECTOR_NEW_STACK(16);
+        pw_dirs = HASH_NEW_STACK(16);
 
-        sObject* str = STRING_NEW_STACK("~/");
-        vector_add(gCompletionArray, 
+        struct passwd* pw;
 
-        sObject* gEditingDir2 = STRING_NEW("");
+        vector_add(gCompletionArray, STRING_NEW_STACK("~/"));
+        pw = getpwuid(getuid());
+        hash_put(pw_dirs, "~/", STRING_NEW_STACK(pw->pw_dir));
 
-        if(string_length(user) == 0) {
-            char* home = getenv("HOME");
-            if(home) {
-                string_put(gEditingDir2, home);
-            }
-            else {
-                struct passwd* pw = getpwuid(getuid());
-                if(pw) {
-                    string_put(gEditingDir2, pw->pw_dir);
-                }
-                else {
-                    string_put(gEditingDir2, "~");
-                }
-            }
-        }
-        else {
-            struct passwd* pw = getpwnam(string_c_str(user));
-            if(pw) {
-                string_put(gEditingDir2, pw->pw_dir);
-            }
-            else {
-                string_put(gEditingDir2, "~");
-                string_push_back(gEditingDir2, string_c_str(user));
-            }
+        while((pw = getpwent()) != NULL) {
+            sObject* str = STRING_NEW_STACK("~");
+            string_push_back(str, pw->pw_name);
+            string_push_back(str, "/");
+            vector_add(gCompletionArray, str);
+
+            hash_put(pw_dirs, string_c_str(str), STRING_NEW_STACK(pw->pw_dir));
         }
 
-        string_push_back(gEditingDir2, p);
-
-        string_put(gEditingDir, string_c_str(gEditingDir2));
-
-        string_delete(gEditingDir2);
-        string_delete(user);
-
-        vector_add(gCompletionArray, string_c_str(candidate));
+        endpwent();
 
         vector_sort(gCompletionArray, name_sort);
 
@@ -329,30 +364,32 @@ static char* tilda_completion(const char* text, int stat)
     }
 
     while(index < vector_size(gCompletionArray)) {
-        char* candidate = vector_item(gCompletionArray, index);
+        char* candidate = string_c_str((sObject*)vector_item(gCompletionArray, index));
         index++;
 
         if(!strncmp(text, candidate, wordlen)) {
-            int len = strlen(candidate);
-            if(len > 2 && candidate[len-2] == '-' && candidate[len-1] == '>') {
-                rl_completion_append_character = 0;
-            }
-            else {
-                rl_completion_append_character = ' ';
-            }
+            rl_completion_append_character = 0;
+            sObject* dir = hash_item(pw_dirs, candidate);
+            //rl_insert_text(string_c_str(dir));
+
             return strdup(candidate);
         }
     }
 
     return NULL;
 }
+
+/*
+#if !HAVE_DECL_ENVIRON
+            extern char **environ;
+#endif
 */
 
-static char* env_completion(const char* text, int stat)
+static char* env_completion(const char* text, int stat_)
 {
     static int index, wordlen;
 
-    if(stat == 0) {
+    if(stat_ == 0) {
         gCompletionArray = VECTOR_NEW_STACK(16);
         sObject* hash = HASH_NEW_STACK(16);
 
@@ -367,7 +404,7 @@ static char* env_completion(const char* text, int stat)
                     if(TYPE(object) == T_STRING || TYPE(object) == T_VECTOR || TYPE(object) == T_HASH) {
                         hash_put(hash, key, object);
 
-                        sObject* candidate = STRING_NEW_STACK("$");
+                        sObject* candidate = STRING_NEW_STACK("");
                         string_push_back(candidate, hash_loop_key(it));
 
                         vector_add(gCompletionArray, string_c_str(candidate));
@@ -380,7 +417,33 @@ static char* env_completion(const char* text, int stat)
             if(current == gRootObject) break;
 
             current = SUOBJECT(current).mParent;
+
+/*
+            char** p;
+            for(p = environ; *p; p++) {
+                char env_name[PATH_MAX];
+
+                char* p2 = env_name;
+                char* p3 = *p;
+
+                while(*p3 != '=') {
+                    *p2++ = *p3++;
+                }
+
+                char* env = getenv(*p);
+                struct stat estat;
+                if(stat(env, &estat) >= 0) {
+                    if(S_ISDIR(estat.st_mode)) {
+                        *p2++ = '/';
+                    }
+                }
+                *p2 = 0;
+
+                vector_add(gCompletionArray, STRING_NEW_STACK(env_name));
+            }
+*/
         }
+
         vector_sort(gCompletionArray, name_sort);
 
         wordlen = strlen(text);
@@ -440,10 +503,11 @@ char** readline_on_complete(const char* text, int start, int end)
 
     int sline = 1;
     gReadlineCurrentObject = gCurrentObject;
-    BOOL result = parse(string_c_str(cmdline), "readline", &sline, gReadlineBlock, &gReadlineCurrentObject);
+    BOOL result = parse(string_c_str(cmdline), "readline", &sline, gReadlineBlock, &gReadlineCurrentObject, FALSE);
 
-    /// in the block?
-    if(!result && (SBLOCK(gReadlineBlock).mCompletionFlags & COMPLETION_FLAGS_BLOCK|COMPLETION_FLAGS_ENV_BLOCK)) {
+
+    /// in the block? get the block
+    if(!result && (SBLOCK(gReadlineBlock).mCompletionFlags & (COMPLETION_FLAGS_BLOCK|COMPLETION_FLAGS_ENV_BLOCK))) {
         while(1) {
             if(SBLOCK(gReadlineBlock).mStatmentsNum > 0) {
                 sStatment* statment = SBLOCK(gReadlineBlock).mStatments + SBLOCK(gReadlineBlock).mStatmentsNum - 1;
@@ -476,24 +540,62 @@ char** readline_on_complete(const char* text, int start, int end)
         }
     }
 
+    /// expand tilda ///
+    if(SBLOCK(gReadlineBlock).mCompletionFlags & COMPLETION_FLAGS_TILDA) {
+        if(SBLOCK(gReadlineBlock).mStatmentsNum > 0) {
+            sStatment* statment = SBLOCK(gReadlineBlock).mStatments + SBLOCK(gReadlineBlock).mStatmentsNum - 1;
+
+            if(statment->mCommandsNum > 0) {
+                sCommand* command = statment->mCommands + statment->mCommandsNum-1;
+
+                char* arg = command->mArgs[command->mArgsNum-1];
+
+                if(command->mArgsNum > 0 && strstr(arg, "/")) {
+                    char* new_arg;
+                    tilda_expasion(arg, ALLOC &new_arg);
+
+                    rl_point -= rl_delete_text(strlen(rl_line_buffer) - strlen(arg), strlen(rl_line_buffer));
+                    if(rl_point < 0) { rl_point = 0; }
+                    rl_insert_text(new_arg);
+
+                    FREE(new_arg);
+                }
+                else {
+                    char** result = rl_completion_matches(text, tilda_completion);
+                    stack_end_stack();
+                    return result;
+                }
+            }
+            else {
+                char** result = rl_completion_matches(text, tilda_completion);
+                stack_end_stack();
+                return result;
+            }
+        }
+        else {
+            char** result = rl_completion_matches(text, tilda_completion);
+            stack_end_stack();
+            return result;
+        }
+    }
+
     if(SBLOCK(gReadlineBlock).mStatmentsNum == 0) {
-        char** result = rl_completion_matches(text, program_completion);
+        char** result = rl_completion_matches(text, all_program_completion);
         stack_end_stack();
         return result;
     }
-    else if(SBLOCK(gReadlineBlock).mStatmentsNum > 0) {
+    else {
         sStatment* statment = SBLOCK(gReadlineBlock).mStatments + SBLOCK(gReadlineBlock).mStatmentsNum - 1;
 
-        if(statment->mCommandsNum == 0) {
-            char** result = rl_completion_matches(text, program_completion);
+        if(statment->mCommandsNum == 0 || SBLOCK(gReadlineBlock).mCompletionFlags & COMPLETION_FLAGS_LAST_CHAR_IS_PIPE) {
+            char** result = rl_completion_matches(text, all_program_completion);
             stack_end_stack();
             return result;
         }
         else {
             sCommand* command = statment->mCommands + statment->mCommandsNum-1;
 
-            const int command_num = command->mArgsNum + (SBLOCK(gReadlineBlock).mCompletionFlags & COMPLETION_FLAGS_WORDBREAK_CHARACTER ? 1:0);
-
+            /// get user completion ///
             sObject* ucompletion;
             if(command->mArgsNum > 0) {
                 sObject* completion_object = gCompletionObject;
@@ -543,13 +645,17 @@ char** readline_on_complete(const char* text, int start, int end)
                 ucompletion = NULL;
             }
 
-            if(SBLOCK(gReadlineBlock).mCompletionFlags & (COMPLETION_FLAGS_ENV)) {
+            char last_char = rl_line_buffer[strlen(rl_line_buffer)-1];
+
+            if(SBLOCK(gReadlineBlock).mCompletionFlags & COMPLETION_FLAGS_ENV) {
                 char** result = rl_completion_matches(text, env_completion);
                 stack_end_stack();
                 return result;
             }
-            else if(command_num < 2 || SBLOCK(gReadlineBlock).mCompletionFlags & (COMPLETION_FLAGS_NEXT_PIPE|COMPLETION_FLAGS_NEXT_STATMENT)) {
-                if(command->mMessagesNum > 0 && !(SBLOCK(gReadlineBlock).mCompletionFlags & (COMPLETION_FLAGS_NEXT_PIPE|COMPLETION_FLAGS_NEXT_STATMENT))) {
+            else if(command->mArgsNum == 0 
+                || command->mArgsNum == 1 && last_char != ' ' && last_char != '"' && last_char != '\'')
+            {
+                if(command->mMessagesNum > 0) {
                     char** result = rl_completion_matches(text, message_completion);
                     stack_end_stack();
                     return result;
@@ -575,7 +681,7 @@ char** readline_on_complete(const char* text, int start, int end)
                 SFUN(fun).mLocalObjects = stackframe;
 
                 sObject* argv = VECTOR_NEW_GC(16, FALSE);
-                if(command->mArgsNum == 1 || (SBLOCK(gReadlineBlock).mCompletionFlags & COMPLETION_FLAGS_WORDBREAK_CHARACTER)) 
+                if(command->mArgsNum == 1 || (SBLOCK(gReadlineBlock).mCompletionFlags & COMPLETION_FLAGS_LAST_CHAR_IS_SPACE)) 
                 {
                     vector_add(argv, STRING_NEW_GC(command->mArgs[0], FALSE));
                     vector_add(argv, STRING_NEW_GC("", FALSE));
@@ -763,7 +869,7 @@ static int readline_macro(int count, int key)
     int rcode = 0;
     sObject* block = BLOCK_NEW_STACK();
     int sline = 1;
-    if(parse("root->macro", "macro", &sline, block, NULL)) {
+    if(parse("root->macro", "macro", &sline, block, NULL, TRUE)) {
         pipes_set_signal();
 
         sObject* fun = FUN_NEW_STACK(NULL);
@@ -809,7 +915,6 @@ static int readline_macro(int count, int key)
 
     return 0;
 }
-
 static int skip_quoted(const char *s, int i, char q)
 {
    while(s[i] && s[i]!=q) 
@@ -821,7 +926,7 @@ static int skip_quoted(const char *s, int i, char q)
    return i;
 }
 
-static int lftp_char_is_quoted(const char *string,int eindex)
+static int lftp_char_is_quoted(const char *string, int eindex)
 {
   int i, pass_next;
 
@@ -840,7 +945,7 @@ static int lftp_char_is_quoted(const char *string,int eindex)
           i = skip_quoted (string, ++i, quote);
           if (i > eindex)
             return 1;
-          i--;  /* the skip functions increment past the closing quote. */
+          i--;
         }
       else if (string[i] == '\\')
         {
@@ -860,8 +965,8 @@ void pipes_readline_init(BOOL runtime_script)
 {
     rl_attempted_completion_function = readline_on_complete;
     rl_completion_entry_function = readline_filename_completion_null_generator;
-    rl_completer_quote_characters = "\"'";
-    rl_completer_word_break_characters = " \t";
+    rl_completer_quote_characters = "\\\"'";
+    rl_completer_word_break_characters = " \t\n\"'|!&;()=$";
     rl_completion_append_character= ' ';
     rl_char_is_quoted_p = (rl_linebuf_func_t*)lftp_char_is_quoted;
 
