@@ -499,6 +499,7 @@ static BOOL wait_child_program(pid_t pid, int nextout2, sRunInfo* runinfo, char*
                     perror("tcsetpgrp(pipes)");
                     exit(1);
                 }
+
                 return FALSE;
             }
             else if(WIFSIGNALED(status)) {  // a pipes external program which is not a last command can't be stopped by CTRL-Z
@@ -838,6 +839,8 @@ BOOL run_function(sObject* fun, sObject* nextin, sObject* nextout, sRunInfo* run
         }
     }
 
+runinfo->mRCode = rcode;
+
     (void)vector_pop_back(gStackFrames);
 
     SFUN(fun).mLocalObjects = local_objects;
@@ -961,6 +964,20 @@ static BOOL run_object(sObject* object, sObject* nextin, sObject* nextout, sRunI
         case T_HASH: { 
             hash_it* it = hash_loop_begin(object);
             while(it) {
+                char* key = hash_loop_key(it);
+                if(!fd_write(nextout, key)) {
+                    sCommand* command = runinfo->mCommand;
+                    err_msg("signal interrupt9", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                    return FALSE;
+                }
+                if(!fd_write(nextout, "\n")) {
+                    sCommand* command = runinfo->mCommand;
+                    err_msg("signal interrupt10", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                    return FALSE;
+                }
+
                 sObject* item = hash_loop_item(it);
 
                 if(!run_object(item, nextin, nextout, runinfo)) {
@@ -1428,7 +1445,7 @@ BOOL load_file(char* fname, sObject* nextin, sObject* nextout, sRunInfo* runinfo
     int rcode = 0;
     sObject* block = BLOCK_NEW_STACK();
     int sline = 1;
-    if(parse(buf, fname, &sline, block, NULL)) {
+    if(parse(buf, fname, &sline, block, NULL, TRUE)) {
         FREE(buf);
 
         sObject* fun = FUN_NEW_STACK(NULL);
