@@ -61,6 +61,10 @@ static void object_delete(sObject* obj)
             memchecker_delete_stack(obj);
             break;
 
+        case T_EXTOBJ:
+            SEXTOBJ(obj).mFreeFun(SEXTOBJ(obj).mObject);
+            break;
+
         default:
             fprintf(stderr,"unexpected err in object_delete stack\n");
             exit(1);
